/*
 * interface data access header
 *
 * $Id$
 */
#ifndef NETSNMP_ACCESS_INTERFACE_CONFIG_H
#define NETSNMP_ACCESS_INTERFACE_CONFIG_H

/*
 * all platforms use this generic code
 */
config_require(if-mib/data_access/interface)

/**---------------------------------------------------------------------*/
/*
 * configure required files
 *
 * Notes:
 *
 * 1) prefer functionality over platform, where possible. If a method
 *    is available for multiple platforms, test that first. That way
 *    when a new platform is ported, it won't need a new test here.
 *
 * 2) don't do detail requirements here. If, for example,
 *    HPUX11 had different reuirements than other HPUX, that should
 *    be handled in the *_hpux.h header file.
 */

#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES

config_exclude(mibII/interfaces)

#   if defined( linux )

    config_require(util_funcs)
    config_require(if-mib/data_access/interface_linux)
    config_require(if-mib/data_access/interface_ioctl)

#   elif defined( openbsd3 ) || \
         defined( freebsd4 ) || defined( freebsd5 ) || defined( freebsd6 ) || \
         defined( darwin )   || defined( dragonfly ) || defined( netbsd1 )

    config_require(if-mib/data_access/interface_sysctl)

#   elif defined( solaris2 )

    config_require(if-mib/data_access/interface_solaris2)

#   else

    config_error(This platform does not yet support IF-MIB rewrites)

#   endif
#else
#   define NETSNMP_ACCESS_INTERFACE_NOARCH 1
#endif

#if PROD_PRODUCT_RBT_SH
/**
 * Map a virtual interface name to the physical interface name if a mapping
 * exists
 * \param virtual_inf               virtual interface name
 * \param ret_physical_inf	    returns physical interface name (if no
 *                                  mapping exists, the "virtual" name is
 *                                  returned)
 */
int snmp_map_virtual_to_physical(char const *virtual_inf,
				 char const **ret_physical_inf);


/**
 * Map a physical interface name to the virtual interface name if a mapping
 * exists
 * \param physical_inf              physical interface name
 * \param ret_virtual_inf	    returns virtual interface name (if no
 *                                  mapping exists, the "physical" name is
 *                                  returned)
 */
int snmp_map_physical_to_virtual(char const *physical_inf,
				 char const **ret_virtual_inf);


/**
 * Mapping of virtual to physical interface names
 */
typedef struct virtual_physical_map_entry {
    char *virtual_inf;
    char *physical_inf;
}snmp_virtual_physical_map_entry;

static const snmp_virtual_physical_map_entry sh_virtual_physical_inf_map[] = 
{{"primary", "prihw"}, {NULL, NULL}};

static const snmp_virtual_physical_map_entry vsp_virtual_physical_inf_map[] = 
{{"primary", "prihw"}, {"aux", "auxhw"}, {NULL, NULL}};
#endif /* PROD_PRODUCT_RBT_SH */
#endif /* NETSNMP_ACCESS_INTERFACE_CONFIG_H */
