/*
 *
 * URL:  $URL$
 * Revision:  $Revision: #1 $
 * Date:      $Date: 2017/11/02 $
 * Author:    $Author: build $
 *
 * (C) Copyright 2002-2012 Riverbed Technology, Inc.
 * All rights reserved.
 *
 */

static const char rcsid[] = "$Id: //depot/prod/cag/R4.4.1x/net-snmp/net-snmp-5.7.1/agent/sn_modules.c#1 $";

#include "common.h"
#include "sn_modules.h"
#include "tpaths.h"

#if PROD_PRODUCT_RBT_SH
#include "rbt_model.h"
#include "file_utils.h"
#endif /* PROD_PRODUCT_RBT_SH */


/* ============================================================================
 * This is SNMP's interface to loading plug-in modules.
 * There are two separate implementations of this API, one for
 * when modules are dynamically linked (preferred), and one for
 * when they are statically linked (required to run Purify).
 */


/* ============================================================================
 * Statically linked
 *
 * When modules are added or removed, both the list of prototypes and
 * the list of calls to initialization functions must be updated.  If
 * the module has a deinitialization function, that must be added too.
 */

#ifdef STATIC_MODULES

typedef struct lc_dso_info lc_dso_info;

/* ========================================================================= */
/* Customer-specific graft point 1: declarations
 * =========================================================================
 */
#ifdef INC_SN_MOD_INC_GRAFT_POINT
#undef SN_MOD_INC_GRAFT_POINT
#define SN_MOD_INC_GRAFT_POINT 1
#include "../bin/snmp/tms/sn_modules.inc.c"
#endif /* INC_SN_MOD_INC_GRAFT_POINT */


/* ------------------------------------------------------------------------- */
int
sn_modules_load(void)
{
    return(0);
}

/* ------------------------------------------------------------------------- */
int
sn_modules_init(void)
{
    int err = 0;

/* ========================================================================= */
/* Customer-specific graft point 2: init
 * =========================================================================
 */

#ifdef INC_SN_MOD_INC_GRAFT_POINT
#undef SN_MOD_INC_GRAFT_POINT
#define SN_MOD_INC_GRAFT_POINT 2
#include "../bin/snmp/tms/sn_modules.inc.c"
#endif /* INC_SN_MOD_INC_GRAFT_POINT */

 bail:
    return(err);
}


/* ------------------------------------------------------------------------- */
int
sn_modules_deinit(void)
{

/* ========================================================================= */
/* Customer-specific graft point 3: deinit
 * =========================================================================
 */

#ifdef INC_SN_MOD_INC_GRAFT_POINT
#undef SN_MOD_INC_GRAFT_POINT
#define SN_MOD_INC_GRAFT_POINT 3
#include "../bin/snmp/tms/sn_modules.inc.c"
#endif /* INC_SN_MOD_INC_GRAFT_POINT */

    return(0);
}


/* ------------------------------------------------------------------------- */
int
sn_modules_unload(void)
{
    return(0);
}


/* ============================================================================
 * Dynamically linked
 */
#else /* STATIC_MODULES */

#include "dso.h"
static const char *sn_module_filename_prefix = "sn_";
static lc_dso_context *sn_module_dso_context = NULL;

/* ------------------------------------------------------------------------- */
int
sn_modules_load(void)
{
    int err = 0;
    char *mod_path = NULL;
#if PROD_PRODUCT_RBT_SH
    const char *mfg_type = NULL;
    char *modules_dir = NULL;
    tbool exists = false;
#endif /* PROD_PRODUCT_RBT_SH */

    err = lc_dso_context_new(&sn_module_dso_context, NULL);
    bail_error_null(err, sn_module_dso_context);

    err = lc_devel_lookup_dir(SNMP_MODULE_PATH, &mod_path);
    bail_error(err);

    /* Load mib modules that are common to all variants. */
    err = lc_dso_module_load_mult(sn_module_dso_context,
				  mod_path,
				  sn_module_filename_prefix,
				  llp_replace_old, false);
    bail_error(err);
 
#if PROD_PRODUCT_RBT_SH
    /* Load mib modules that are specific to this variant. */
    err = lmr_get_model_mfg_type(&mfg_type);
    bail_error(err);

    modules_dir = smprintf("/opt/tms/variants/%s/lib/snmp/modules",
                           mfg_type);

    err = lf_test_exists(modules_dir, &exists);
    bail_error(err);

    if (exists) {
        lc_log_debug(LOG_DEBUG, "Loading modules from %s", modules_dir);
        err = lc_dso_module_load_mult(sn_module_dso_context, modules_dir,
                                      sn_module_filename_prefix,
                                      llp_replace_old, false);
        bail_error(err);
    }
#endif /* PROD_PRODUCT_RBT_SH */

 bail:
#if PROD_PRODUCT_RBT_SH
    safe_free(modules_dir);
#endif /* PROD_PRODUCT_RBT_SH */
    safe_free(mod_path);
    return(err);
}


/* ------------------------------------------------------------------------- */
int
sn_modules_init(void)
{
    int err = 0;

    err = lc_dso_module_init_all(sn_module_dso_context, true, true, NULL);
    complain_error(err);
    err = 0;

 bail:
    return(err);
}


/* ------------------------------------------------------------------------- */
int
sn_modules_deinit(void)
{
    int err = 0;

    err = lc_dso_module_deinit_all(sn_module_dso_context, false, true, NULL);
    bail_error(err);

 bail:
    return(err);
}


/* ------------------------------------------------------------------------- */
int
sn_modules_unload(void)
{
    int err = 0;

    err = lc_dso_module_unload_all(sn_module_dso_context);
    bail_error(err);

    err = lc_dso_context_free(&sn_module_dso_context);
    bail_error(err);

 bail:
    return(err);
}

#endif /* STATIC_MODULES */
