/*
 *
 * URL:       $URL$
 * Revision:  $Revision: #1 $
 * Date:      $Date: 2017/11/02 $
 * Author:    $Author: build $
 *
 * (C) Copyright 2002-2012 Riverbed Technology, Inc.
 * All rights reserved.
 *
 */

static const char rcsid[] = "$Id: //depot/prod/cag/R4.4.1x/net-snmp/net-snmp-5.7.1/agent/sn_utils.c#1 $";

#include "common.h"
#include "bnode.h"
#include "sn_main.h"
#include "sn_mgmt.h"
#include "sn_utils.h"
#include "md_client.h"
#include "bswap.h"


/* ------------------------------------------------------------------------- */
int
sn_oid_concat(oid *ret_full_oid, uint8 full_oid_max_len,
	      uint8 *ret_full_oid_len, uint8 num_oids,
	      oid *oid1, uint8 oid1_len, ...)
{
    int err = 0;
    va_list ap;
    oid *next_oid = NULL;
    uint8 next_oid_len = 0;
    
    bail_null(ret_full_oid);
    bail_null(ret_full_oid_len);
    bail_require(num_oids >= 1);
    bail_require_msg_custom(full_oid_max_len >= oid1_len, 
			    lc_err_no_buffer_space, "OID too long");

    memcpy(ret_full_oid, oid1, oid1_len * sizeof(oid));
    *ret_full_oid_len = oid1_len;

    if (num_oids == 1) {
	goto bail;
    }

    va_start(ap, oid1_len);
    while (--num_oids > 0) {
	next_oid = va_arg(ap, oid *);
	next_oid_len = va_arg(ap, int);
	bail_require_msg_custom(*ret_full_oid_len + next_oid_len <=
				full_oid_max_len, lc_err_no_buffer_space,
				"OID too long");
	memcpy(ret_full_oid + *ret_full_oid_len, next_oid,
	       next_oid_len * sizeof(oid));
	*ret_full_oid_len += next_oid_len;
    }
    va_end(ap);

 bail:
    return(err);
}


/* ------------------------------------------------------------------------- */
int
sn_get_snmp_value_by_name(const char *node_name, const char *db_name,
        bn_type expected_type, void *ret_result, size_t result_max_size, 
        size_t *ret_result_size, u_char *ret_snmp_type)

{
    int err = 0;
    bn_binding *binding = NULL;

    err = mdc_get_binding(NULL, NULL, true, &binding, node_name, db_name);
    bail_error(err);

    if (binding) {
	err = sn_binding_to_snmp_result
	    (binding, expected_type, ret_result, result_max_size, 
	     ret_result_size, ret_snmp_type);
	bail_error(err);
    }

 bail:
    bn_binding_free(&binding);
    return(err);
}

/* ------------------------------------------------------------------------- */
int
sn_binding_to_snmp_result(const bn_binding *binding, bn_type expected_type,
			  void *ret_result, size_t result_max_size, 
			  size_t *ret_result_size, u_char *ret_snmp_type)
{
    int err = 0;
    bn_type type = bt_NONE;
    const tstring *name = NULL;
    void *result = NULL;
    size_t result_size = 0;
    u_char snmp_type = 0;

    int8 int8_return = 0;
    uint8 uint8_return = 0;
    int16 int16_return = 0;
    uint16 uint16_return = 0;
    int32 int32_return = 0;
    uint32 uint32_return = 0;
    int64 int64_return = 0;
    uint64 uint64_return = 0;
    tbool bool_return = 0;
    struct counter64 counter64_return;
    lt_dur_sec dur_sec_return = 0;
    lt_dur_ms dur_ms_return = 0;
    lt_dur_us dur_us_return = 0;
    lt_time_sec datetime_sec = 0;
    lt_time_ms datetime_ms = 0;
    lt_time_us datetime_us = 0;
    ipv6addr ipv6addr_return;
    char *str_return = NULL;
    char *sp = NULL;
    uint32 oid_return[MAX_OID_LEN];
    uint32 oid_length = 0;

    bail_null(binding);
    bail_null(ret_result);
    bail_null(ret_result_size);

    err = bn_binding_get_type(binding, ba_value, &type, NULL);
    bail_error_require(err, expected_type == bt_NONE || expected_type == type);
    err = bn_binding_get_name(binding, &name);
    bail_error(err);

    switch (type) {

    case bt_int8:
	err = bn_binding_get_int8(binding, ba_value, NULL, &int8_return);
	bail_error(err);
	int32_return = int8_return;
	result = &int32_return;
	result_size = sizeof(int32);
	snmp_type = ASN_INTEGER;
	break;

    case bt_uint8:
	err = bn_binding_get_uint8(binding, ba_value, NULL, &uint8_return);
	bail_error(err);
	uint32_return = uint8_return;
	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_UNSIGNED;
	break;

    case bt_int16:
	err = bn_binding_get_int16(binding, ba_value, NULL, &int16_return);
	bail_error(err);
	int32_return = int16_return;
	result = &int32_return;
	result_size = sizeof(int32);
	snmp_type = ASN_INTEGER;
	break;

    case bt_uint16:
	err = bn_binding_get_uint16(binding, ba_value, NULL, &uint16_return);
	bail_error(err);
	uint32_return = uint16_return;
	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_UNSIGNED;
	break;

    case bt_int32:
	err = bn_binding_get_int32(binding, ba_value, NULL, &int32_return);
	bail_error(err);
	result = &int32_return;
	result_size = sizeof(int32);
	snmp_type = ASN_INTEGER;
	break;

    case bt_uint32:
	err = bn_binding_get_uint32(binding, ba_value, NULL, &uint32_return);
	bail_error(err);
	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_UNSIGNED;
	break;

    case bt_bool:
	err = bn_binding_get_tbool(binding, ba_value, NULL, &bool_return);
	bail_error(err);
	uint32_return = bool_return;
	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_UNSIGNED;
	break;

    case bt_ipv4addr:
	err = bn_binding_get_ipv4addr(binding, ba_value, NULL, &uint32_return);
	bail_error(err);

	/*
	 * XXX/EMT: not sure if this is right on all architectures
	 */
	uint32_return = htonl(uint32_return);

	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_IPADDRESS;
	break;

    case bt_ipv6addr:
        err = bn_binding_get_ipv6addr(binding, ba_value, NULL, &ipv6addr_return);
        bail_error(err);

        if (lc_is_ipv4_mapped_ipv6addr(ipv6addr_return)) {
	    err = bn_binding_get_ipv4addr(binding, ba_value, NULL, &uint32_return);
	    bail_error(err);
	    uint32_return = htonl(uint32_return);

	    result = &uint32_return;
	    result_size = sizeof(uint32);
	    snmp_type = ASN_IPADDRESS;
        } else {
            /* XXX/abutala: Currently, only ipv4 addresses are returned if the node
             * with bt_ipv6addr as binding type is queried by SNMP. Such nodes with
             * ipv6 addresses are silently ignored.
             * In future, such nodes should be handled to return InetAddress and Inet
             * -AddressType values, and OIDs should be registered with these two new
             * address types for compatibility.
             * When this new support is added, backwards compatibility should be
             * maintained for OIDs which are relying on nodes with bt_ipv6addr but
             * still are registered with IpAddress as their datatype. One such
             * example is peerAddress OID. Hardcoding may be needed in that case.
             */
        }
	break;

    case bt_duration_sec:
	err = bn_binding_get_duration_sec(binding, ba_value, NULL,
					  (lt_dur_sec *)&dur_sec_return);
	bail_error(err);
	uint32_return = dur_sec_return * 100;

#if 0
 	/*
	 * If we wanted to clamp it rather than wrap around, use this
	 * instead.  But note that SNMP expects the TimeTicks type to
	 * wrap, so clamping probably isn't the right behavior.
	 */
	uint32_return = clamp(dur_sec_return, 0, UINT32_MAX / 100) * 100;
#endif

	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_TIMETICKS;
	break;

    case bt_duration_ms:
	err = bn_binding_get_duration_ms(binding, ba_value, NULL,
					 (lt_dur_ms *)&dur_ms_return);
	bail_error(err);
	uint32_return = dur_ms_return / 10;
#if 0
	uint32_return = clamp(dur_ms_return, 0, (uint64)UINT32_MAX * 10) / 10;
#endif

	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_TIMETICKS;
	break;

    case bt_duration_us:
	err = bn_binding_get_duration_us(binding, ba_value, NULL,
					 (lt_dur_us *)&dur_us_return);
	bail_error(err);
	uint32_return = dur_us_return / 10000;
#if 0
	uint32_return = clamp(dur_us_return, 0,
			      (uint64)UINT32_MAX * 10000) / 10000;
#endif
	result = &uint32_return;
	result_size = sizeof(uint32);
	snmp_type = ASN_TIMETICKS;
	break;

    case bt_oid:
	bail_require(sizeof(oid) == sizeof(uint32));
	err = bn_binding_get_oid
	    (binding, ba_value, NULL, MAX_OID_LEN, oid_return, &oid_length);
	bail_error(err);
	result = oid_return;
	result_size = sizeof(uint32) * oid_length;
	snmp_type = ASN_OBJECT_ID;
	break;

    case bt_datetime_sec:
	err = bn_binding_get_datetime_sec(binding, ba_value, NULL,
					  &datetime_sec);
	bail_error(err);

	result = (u_char *) date_n_time((time_t *) &datetime_sec,
					&result_size);

	snmp_type = ASN_OCTET_STR;
	break;

    case bt_datetime_ms:
	err = bn_binding_get_datetime_ms(binding, ba_value, NULL,
					  &datetime_ms);
	bail_error(err);

	datetime_sec = datetime_ms / 1000;

	result = (u_char *) date_n_time((time_t *) &datetime_sec,
					&result_size);

	snmp_type = ASN_OCTET_STR;
	break;

    case bt_datetime_us:
	err = bn_binding_get_datetime_us(binding, ba_value, NULL,
					 &datetime_us);
	bail_error(err);

	datetime_sec = datetime_us / (1000 * 1000);

	result = (u_char *) date_n_time((time_t *) &datetime_sec,
					&result_size);

	snmp_type = ASN_OCTET_STR;
	break;

    case bt_time_sec:
	/* XXX is there a time of day */
	err = bn_binding_get_time_sec(binding, ba_value, NULL,
				      &datetime_sec);
	bail_error(err);

	result = (u_char *) date_n_time((time_t *) &datetime_sec,
					&result_size);

	snmp_type = ASN_OCTET_STR;
	break;

    case bt_time_ms:
	err = bn_binding_get_time_ms(binding, ba_value, NULL,
				      &datetime_ms);
	bail_error(err);

	datetime_sec = datetime_ms / 1000;

	result = (u_char *) date_n_time((time_t *) &datetime_sec,
					&result_size);

	snmp_type = ASN_OCTET_STR;
	break;

    case bt_time_us:
	err = bn_binding_get_time_us(binding, ba_value, NULL,
				      &datetime_us);
	bail_error(err);

	datetime_sec = datetime_us / (1000 * 1000);

	result = (u_char *) date_n_time((time_t *) &datetime_sec,
					&result_size);

	snmp_type = ASN_OCTET_STR;
	break;


    case bt_uint64:
	err = bn_binding_get_uint64(binding, ba_value, NULL, &uint64_return);
	bail_error(err);

	counter64_return.high = uint64_return >> 32;
	counter64_return.low = uint64_return & 0xFFFFFFFF;

	result = &counter64_return;
	result_size = sizeof(counter64_return);
	snmp_type = ASN_COUNTER64;
	break;


    /*
     * There is no SNMP standard for a Gauge64 at present (merely
     * Counter64). So return as a string. One could do something
     * along the lines of the textual conventions given in rfc2856.
    */
    case bt_int64:
    case bt_char:
    case bt_ipv4prefix:
    case bt_macaddr802:
    case bt_string:
    case bt_link:
    case bt_name:
    case bt_hostname:
    case bt_uri:
    case bt_charlist:
    case bt_regex:
    case bt_globpattern:
        err = bn_binding_get_str
            (binding, ba_value, bt_NONE, NULL, &str_return);
        bail_error_null(err, str_return);
        result = str_return;
        result_size = strlen(result);
        snmp_type = ASN_OCTET_STR;
	break;

    default:
	lc_log_debug(LOG_ERR, "Binding %s has unsupported type %s",
		     ts_str(name), lc_enum_to_string(bn_type_map, type));
	break;

    }

    if (result_size > result_max_size) {
	lc_log_debug(LOG_WARNING, "Binding %s was too big to fit in return "
		     "buffer of size %" LPRI_d_SIZE_T " -- truncating",
		     ts_str(name), result_max_size);
	result_size = result_max_size;
    }

    memcpy(ret_result, result, result_size);
    *ret_result_size = result_size;
    
    if (ret_snmp_type) {
	*ret_snmp_type = snmp_type;
    }

 bail:
    safe_free(str_return);
    return(err);
}


/* ------------------------------------------------------------------------- */
int
sn_free_var_list(netsnmp_variable_list **inout_vars)
{
    int err = 0;
    netsnmp_variable_list *var = NULL, *next_var = NULL;

    if (inout_vars) {
	var = *inout_vars;
	while (var) {
	    next_var = var->next_variable;

        if (var->val.string != var->buf) 
        {
            safe_free(var->val.string);
        }

	    safe_free(var->name);
	    safe_free(var);
	    var = next_var;
	}
	*inout_vars = NULL;
    }

 bail:
    return(err);
}
