/*
 *
 * URL:       $URL$
 * Revision:  $Revision: #1 $
 * Date:      $Date: 2017/11/02 $
 * Author:    $Author: build $
 *
 * (C) Copyright 2002-2012 Riverbed Technology, Inc.
 * All rights reserved.
 *
 */

#ifndef __SN_UTILS_H_
#define __SN_UTILS_H_

static const char *rcsid_h__SN_UTILS_H_ __attribute__((unused)) =
    "$Id: //depot/prod/cag/R4.4.1x/net-snmp/net-snmp-5.7.1/include/net-snmp/agent/sn_utils.h#1 $";

#ifdef __cplusplus
extern "C" {
#endif
#if 0 /* for emacs */
}
#endif


/* ------------------------------------------------------------------------- */
/* Copy one or more OIDs while concatenating them together.
 * They are copied into the pre-allocated buffer provided.
 */
int sn_oid_concat(oid *ret_full_oid, uint8 full_oid_max_len,
		  uint8 *ret_full_oid_len, uint8 num_oids,
		  oid *oid1, uint8 oid1_len, ...);


/* ------------------------------------------------------------------------- */
/* Get the value of a management node specified by name, in the form
 * of an SNMP return value.  The result is copied into the buffer
 * provided.  This is a thin wrapper around mdc_get_binding() and
 * sn_binding_to_snmp_result().
 */
int sn_get_snmp_value_by_name(const char *node_name, const char *db_name,
        bn_type expected_type, void *ret_result, size_t result_max_size, 
        size_t *ret_result_size, u_char *ret_snmp_type);


/* ------------------------------------------------------------------------- */
/* Convert a binding to an SNMP data type in the form required to return
 * from NET-SNMP get handler functions.
 *
 * \param binding The binding whose data to convert.
 *
 * \param expected_type If the caller thinks he knows what type the
 * binding is, this can be specified as a sanity check.  Otherwise,
 * pass bt_NONE.
 *
 * \param result_max_len The size of the buffer passed for ret_result.
 * No more bytes than this will be filled in.  If the result exceeds
 * this length, it will be truncated and a warning logged.
 *
 * \retval ret_result The result of the conversion is placed into this
 * pre-allocated buffer.
 *
 * \retval ret_result_len The length of the result in bytes.  This
 * will always be less than or equal to result_max_len.
 *
 * \retval ret_snmp_type The ASN.1 data type of the returned value.
 * This may be NULL if the caller doesn't care about the type.
 */
int sn_binding_to_snmp_result(const bn_binding *binding, bn_type expected_type,
			      void *ret_result,size_t result_max_len, 
			      size_t *ret_result_len, u_char *ret_snmp_type);


/* ------------------------------------------------------------------------- */
/* Free a dynamically-allocated linked list of netsnmp_variable_list 
 * structures.
 */
int sn_free_var_list(netsnmp_variable_list **inout_vars);


/*
 * This matches the number on line 199 of net-snmp/agent/snmp_vars.c,
 * which is not currently exported from any of its header files.
 */
static const uint32 sn_return_buf_size = 256;

#ifdef __cplusplus
}
#endif

#endif /* __SN_UTILS_H_ */
